use axum::Router;
use tower_http::services::ServeDir;
use tracing_subscriber::{layer::SubscriberExt, util::SubscriberInitExt};

use crate::util::AppState;

mod components;
mod database;
mod icons;
mod layouts;
mod models;
mod pages;
mod util;

pub use util::error::AppError;

#[tokio::main]
async fn main() {
    tracing_subscriber::registry()
        .with(
            tracing_subscriber::EnvFilter::try_from_default_env()
                .unwrap_or_else(|_| format!("{}=debug", env!("CARGO_CRATE_NAME")).into()),
        )
        .with(tracing_subscriber::fmt::layer())
        .init();

    dotenvy::dotenv().expect("Could not initialize dotenvy");

    // Connect to databse
    let pool = database::connect_database().await;

    // Migrate
    sqlx::migrate!("./migrations")
        .run(&pool)
        .await
        .expect("Database migrations failed");

    // Initialize global app state
    let app_state = AppState { pool };

    // build our application with a route
    let app = Router::new()
        .nest_service("/assets", ServeDir::new("assets"))
        .merge(pages::routes())
        .with_state(app_state);

    // run it
    let listener = tokio::net::TcpListener::bind("127.0.0.1:3000")
        .await
        .unwrap();
    println!("listening on {}", listener.local_addr().unwrap());
    axum::serve(listener, app).await.unwrap();
}
