use crate::{
    AppError, layouts,
    models::{exercises::ExerciseVariant, workouts::WorkoutExerciseFull},
    util::AppState,
};
use axum::{
    Router,
    extract::{Path, State},
    routing::get,
};
use maud::{Markup, PreEscaped, html};
use uuid::Uuid;

pub fn routes() -> Router<AppState> {
    Router::new().route("/", get(page))
}

async fn page(State(state): State<AppState>, Path(id): Path<Uuid>) -> Result<Markup, AppError> {
    let exercises = sqlx::query_as!(
        WorkoutExerciseFull,
        r#"
            SELECT exercises.exercise_id, exercise_type as "exercise_type:ExerciseVariant", sets, reps, time, name, description, position
            FROM workout_exercises
            JOIN exercises ON workout_exercises.exercise_id = exercises.exercise_id
            WHERE workout_id = $1
            ORDER BY position
        "#,
        &id,
    ).fetch_all(&state.pool).await?;

    let exercises_json = serde_json::to_string(&exercises).unwrap();

    let content = html! {
        script {
            "window.exerciseData = " (PreEscaped(exercises_json)) ";"
        }
        div x-data="exercisePlayer" {
            @for exercise in exercises {
                (display_exercise(exercise))
            }
        }
    };

    Ok(layouts::desktop(content, "Working out", true))
}

fn display_exercise(exercise: WorkoutExerciseFull) -> Markup {
    html! {
        div x-bind:class={"{ 'hidden': currentPosition !="(exercise.position)" }"} {
            div class="flex justify-center items-center h-48" {
                @match exercise.exercise_type {
                    ExerciseVariant::Time => div {
                        div {
                            span class="font-bold text-6xl" x-text="timeLeft" {}
                            span class="" { " / " }
                            span class="" {(exercise.time.unwrap_or(-1))}
                        }
                    },
                    ExerciseVariant::Number => div {
                        div { span class="font-bold text-6xl" {(exercise.sets.unwrap_or(-1))} " sets" }
                        div { span class="font-bold text-6xl" {(exercise.reps.unwrap_or(-1))} " reps" }
                    },
                    ExerciseVariant::Failure => div {
                        div { span class="font-bold text-6xl" { "Failure" } }
                    }
                }
            }
            div class="flex justify-center gap-3" {
                button class="btn" x-on:click="previous()" { "Previous" }
                button class="btn" disabled[exercise.time.is_none()] x-on:click="pause()" { "Pause" }
                button class="btn" x-on:click="next()" { "Next" }
            }
            div class="mt-5" {
                h2 class="text-center font-bold text-4xl" { (exercise.name) }
                p class="text-center desc" { (exercise.description) }
            }
        }
    }
}
